@@ -60,9 +60,8 @@ pub fn decode<T: ?Sized>(mut datas: Vec<Bson>) -> Result<T, crate::Error>
 pub fn decode_one<T>(type_name: &str, datas: &mut Vec<Bson>) -> Result<T, crate::Error>
     where
         T: DeserializeOwned, {
-    let doc_len = datas.len();
     //decode struct
-    if doc_len > 1 {
+    if datas.len() > 1 {
         return Result::Err(Error::from(format!("[rbatis] rows.rows_affected > 1,but decode one type ({})!", type_name)));
     }
     //single try decode
@@ -71,8 +70,10 @@ pub fn decode_one<T>(type_name: &str, datas: &mut Vec<Bson>) -> Result<T, crate:
     }
     let mut v = Bson::Null;
     let m = datas.remove(0);
+    let mut doc_len = 0;
     match &m {
         Bson::Document(doc) => {
+            doc_len = doc.len();
             if doc_len == 1 {
                 for (k, _v) in doc {
                     v = _v.clone();
