@@ -1,4 +1,4 @@
-use serde_json::{json, Value};
+use bson::{bson, Bson};
 use sqlx_core::column::Column;
 use sqlx_core::decode::Decode;
 use sqlx_core::error::BoxDynError;
@@ -10,60 +10,60 @@ use sqlx_core::value::ValueRef;
 
 use crate::convert::{JsonCodec, RefJsonCodec, ResultCodec};
 
-use crate::{new_json_option_into};
-use crate::types::DateStringCheck;
+use crate::{to_bson_macro};
+use crate::db::db_adapter::DataDecoder;
+use crate::types::Decimal;
 
 impl<'r> JsonCodec for sqlx_core::mssql::MssqlValueRef<'r> {
-    fn try_to_json(self) -> crate::Result<serde_json::Value> {
+    fn try_to_bson(self) -> crate::Result<Bson> {
         //TODO batter way to match type replace use string match
         match self.type_info().name() {
             "NULL" => {
-                return Ok(serde_json::Value::Null);
+                return Ok(Bson::Null);
             }
             "TINYINT" => {
                 let r: Option<i8> = Decode::<'_, Mssql>::decode(self)?;
-                return Ok(new_json_option_into!(r));
+                if let Some(r)=r{
+                    return Ok(bson!(r as i32));
+                }
+                return Ok(Bson::Null);
             }
             "SMALLINT" => {
                 let r: Option<i16> = Decode::<'_, Mssql>::decode(self)?;
-                return Ok(new_json_option_into!(r));
+                if let Some(r)=r{
+                    return Ok(bson!(r as i32));
+                }
+                return Ok(Bson::Null);
             }
             "INT" => {
                 let r: Option<i32> = Decode::<'_, Mssql>::decode(self)?;
-                return Ok(new_json_option_into!(r));
+                return Ok(to_bson_macro!(r));
             }
             "BIGINT" => {
                 let r: Option<i64> = Decode::<'_, Mssql>::decode(self)?;
-                return Ok(new_json_option_into!(r));
+                return Ok(to_bson_macro!(r));
             }
             "REAL" => {
                 let r: Option<f32> = Decode::<'_, Mssql>::decode(self)?;
-                return Ok(new_json_option_into!(r));
+                return Ok(to_bson_macro!(r));
             }
             "FLOAT" => {
                 let r: Option<f64> = Decode::<'_, Mssql>::decode(self)?;
-                return Ok(new_json_option_into!(r));
+                return Ok(to_bson_macro!(r));
             }
 
             "VARCHAR" | "NVARCHAR" | "BIGVARCHAR" | "CHAR" | "BIGCHAR" | "NCHAR" => {
                 let r: Option<String> = Decode::<'_, Mssql>::decode(self)?;
-                return Ok(new_json_option_into!(r));
+                return Ok(to_bson_macro!(r));
             }
 
             "NEWDECIMAL" => {
                 let r: Option<String> = Decode::<'_, Mssql>::decode(self)?;
-                return Ok(json!(r));
+                return Ok(to_bson_macro!(r));
             }
 
-            //TODO
-            // "DATE" | "TIME" | "DATETIME" | "TIMESTAMP" => {
-            //     let r: Option<Vec<u8>> = Decode::<'_, Mssql>::decode(self)?;
-            //     if let Some(mut v) = r {
-            //         return Ok(serde_json::json!(v));
-            //     }
-            //     return Ok(serde_json::Value::Null);
-            // }
-            // you can use already supported types to decode this
+            //TODO "DATE" | "TIME" | "DATETIME" | "TIMESTAMP" => {}
+            // you can use types package to save date to string
             _ => {
                 return Err(crate::Error::from(format!(
                     "un support database type for:{:?}!",
@@ -75,18 +75,20 @@ impl<'r> JsonCodec for sqlx_core::mssql::MssqlValueRef<'r> {
 }
 
 impl RefJsonCodec for Vec<MssqlRow> {
-    fn try_to_json(&self) -> crate::Result<serde_json::Value> {
+    fn try_to_bson(&self,decoder: &dyn DataDecoder) -> crate::Result<Bson> {
         let mut arr = Vec::with_capacity(self.len());
         for row in self {
-            let mut m = serde_json::Map::new();
+            let mut m = bson::Document::new();
             let columns = row.columns();
             for x in columns {
                 let key = x.name();
                 let v: MssqlValueRef = row.try_get_raw(key)?;
-                m.insert(key.to_owned(), v.try_to_json()?);
+                let mut bson=v.try_to_bson()?;
+                decoder.decode(key,&mut bson)?;
+                m.insert(key.to_owned(), bson);
             }
-            arr.push(serde_json::Value::Object(m));
+            arr.push(Bson::Document(m));
         }
-        Ok(serde_json::Value::from(arr))
+        Ok(Bson::from(arr))
     }
 }
